var _interopRequireWildcard = function (obj) { return obj && obj.__esModule ? obj : { 'default': obj }; };

var _toConsumableArray = function (arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } };

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.findModules = findModules;

var _parse = require('acorn');

var _umd = require('acorn-umd');

var _umd2 = _interopRequireWildcard(_umd);

var _estraverse = require('estraverse');

var _estraverse2 = _interopRequireWildcard(_estraverse);

var _lodash$compact$flatten$includes$map$reject = require('lodash');

var _lodash$compact$flatten$includes$map$reject2 = _interopRequireWildcard(_lodash$compact$flatten$includes$map$reject);

var _updateReferences = require('./updateReferences');

var _updateReferences2 = _interopRequireWildcard(_updateReferences);

var _Error = require('./Error');

var _Error2 = _interopRequireWildcard(_Error);

var acornOptions = {
  ecmaVersion: 6,

  sourceType: 'module',

  // Be super loose (as who cares for this purpose)
  allowImportExportEverywhere: true,
  allowReturnOutsideFunction: true,
  allowHashBang: true
};

function findModules(path, _ref) {
  var imports = _ref.imports;
  var scope = _ref.scope;

  var result = [];
  _estraverse2['default'].traverse(scope, {
    enter: function enter(node) {
      switch (node.type) {
        case 'MemberExpression':
          if (_lodash$compact$flatten$includes$map$reject.includes(imports, node.object.name)) {
            if (node.computed) {
              var msg = 'Could not id computed function ' + node.object.name + '[' + node.property.name + ']';
              throw new _Error2['default'](msg, path);
            }
            result.push(node.property.name);
          }
          break;
        case 'CallExpression':
          {
            // Detect chaining
            var callee = node;
            var props = [];
            while (callee = callee.callee) {
              if (callee.property) {
                props.push(callee.property.name);
              }
              if (callee.object) {
                callee = callee.object;
              }
              if (!callee.callee) {
                break;
              }
            }
            if (callee && _lodash$compact$flatten$includes$map$reject.includes(imports, callee.name)) {
              result.push.apply(result, props);
            }
            break;
          }
      }
    }
  });
  return result;
}

exports['default'] = function (code, path, options) {
  var ast = _parse.parse(code, _lodash$compact$flatten$includes$map$reject2['default'].assign({
    ranges: true,
    locations: true
  }, acornOptions, _lodash$compact$flatten$includes$map$reject2['default'].result(options, 'acorn')));

  var result = [];

  // imports to consider lodash (e.g. lodash-compact, lodash, etc)
  var lodashOptions = _lodash$compact$flatten$includes$map$reject.compact(_lodash$compact$flatten$includes$map$reject.flatten([options.lodash, options.output]));

  _lodash$compact$flatten$includes$map$reject2['default'](_umd2['default'](ast, {
    amd: false,
    cjs: _lodash$compact$flatten$includes$map$reject.includes(options.format, 'cjs'),
    es6: _lodash$compact$flatten$includes$map$reject.includes(options.format, 'es6')
  })).filter(function (node) {
    // consider adding lodash-fp & others
    return _lodash$compact$flatten$includes$map$reject.includes(lodashOptions, node.source.value);
  }).each(function (node) {
    // Add direct specifiers (`import {map, pick} from 'lodash'`)
    _lodash$compact$flatten$includes$map$reject2['default'](node.specifiers).map('imported').compact().each(function (requireNode) {
      result.push(requireNode.name);
    }).value();
  }).tap(function (nodes) {
    if (options.update && nodes.length) {
      _updateReferences2['default'](code, path, nodes, options);
    }
  }).map(function (node) {
    // filter the specifiers down to the direct imports
    // (handles `import x,{y,z} from 'foo';)
    return {
      imports: _lodash$compact$flatten$includes$map$reject.reject(node.specifiers, 'imported').map(function (x) {
        return x.local.name;
      }),
      scope: node.scope.block
    };
  }).each(function (node) {
    result.push.apply(result, _toConsumableArray(findModules(path, node)));
  }).value();

  _lodash$compact$flatten$includes$map$reject2['default'](_umd2['default'](ast, {
    amd: _lodash$compact$flatten$includes$map$reject.includes(options.format, 'amd'),
    cjs: false,
    es6: false
  })).map(function (define) {
    var imports = define.imports.filter(function (specifier) {
      return _lodash$compact$flatten$includes$map$reject.includes(lodashOptions, specifier[0].value);
    });
    if (imports.length) {
      return {
        imports: imports,
        scope: define.scope,
        type: define.type
      };
    }
  }).compact().tap(function (nodes) {
    var imports = _lodash$compact$flatten$includes$map$reject.flatten(nodes.map(function (nodes) {
      return _lodash$compact$flatten$includes$map$reject.map(nodes.imports, 0);
    }));
    if (options.update && imports.length) {
      _updateReferences2['default'](code, path, imports, options, nodes[0].type);
    }
  }).map(function (node) {
    // filter the specifiers down to the direct imports
    // (handles `import x,{y,z} from 'foo';)
    return {
      imports: node.imports.map(function (zip) {
        return zip[1].name;
      }),
      scope: node.scope.block
    };
  }).each(function (node) {
    result.push.apply(result, _toConsumableArray(findModules(path, node)));
  }).value();

  if (typeof options.global === 'string') {
    result.push.apply(result, _toConsumableArray(findModules(path, {
      imports: [options.global],
      scope: ast
    })));
  }

  return result;
};